@@ -1,19 +1,22 @@
 # Liblouis Python ctypes bindings
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
-# License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# Copyright (C) 2009, 2010 James Teh <jamie@jantrid.net>
 #
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
-#
-# You should have received a copy of the GNU Library General Public
-# License along with this library; if not, write to the
-# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
-# Boston MA  02110-1301 USA.
+# This file is part of liblouis.
+# 
+# liblouis is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published
+# by the Free Software Foundation, either version 2.1 of the License, or
+# (at your option) any later version.
+# 
+# liblouis is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+# Lesser General Public License for more details.
+# 
+# You should have received a copy of the GNU Lesser General Public
+# License along with liblouis. If not, see <http://www.gnu.org/licenses/>.
+# 
 
 """Liblouis Python ctypes bindings
 These bindings allow you to use the liblouis braille translator and back-translator library from within Python.
@@ -29,6 +32,7 @@ function for information about how liblouis searches for these tables.
 @author: James Teh <jamie@jantrid.net>
 @author: Eitan Isaacson <eitan@ascender.com>
 @author: Michael Whapples <mwhapples@aim.com>
+@author: Davy Kager <mail@davykager.nl>
 """
 
 from ctypes import *
@@ -41,6 +45,10 @@ def _createTablesString(tablesList):
     """Creates a tables string for liblouis calls"""
     return b",".join([x.encode("ASCII") if isinstance(x, str) else bytes(x) for x in tablesList])
 
+def _createTypeformbuf(length, typeform=None):
+    """Creates a typeform buffer for liblouis calls"""
+    return (c_ushort*length)(*typeform) if typeform else (c_ushort*length)()
+
 createStr = None
 if sys.version_info[0] == 2:
     createStr = lambda x: unicode(x)
@@ -62,7 +70,7 @@ try:
 except NameError:
     # Unix/Cygwin
     _loader = cdll
-liblouis = _loader["liblouis.so.4.0"]
+liblouis = _loader["liblouis.so.5.0"]
 
 atexit.register(liblouis.lou_free)
 
@@ -70,27 +78,31 @@ liblouis.lou_version.restype = c_char_p
 
 liblouis.lou_translateString.argtypes = (
     c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p, 
-         POINTER(c_int), POINTER(c_char), POINTER(c_char), c_int)
+         POINTER(c_int), POINTER(c_ushort), POINTER(c_char), c_int)
 
 liblouis.lou_translate.argtypes = (
     c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p, 
-         POINTER(c_int), POINTER(c_char), POINTER(c_char), 
+         POINTER(c_int), POINTER(c_ushort), POINTER(c_char), 
          POINTER(c_int), POINTER(c_int), POINTER(c_int), c_int)
 
 liblouis.lou_backTranslateString.argtypes = (
          c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p,
-         POINTER(c_int), POINTER(c_char), POINTER(c_char), c_int)
+         POINTER(c_int), POINTER(c_ushort), POINTER(c_char), c_int)
 
 liblouis.lou_backTranslate.argtypes = (
          c_char_p, c_wchar_p, POINTER(c_int), c_wchar_p, POINTER(c_int),
-         POINTER(c_char), POINTER(c_char), POINTER(c_int), POINTER(c_int),
+         POINTER(c_ushort), POINTER(c_char), POINTER(c_int), POINTER(c_int),
          POINTER(c_int), c_int)
 
 liblouis.lou_hyphenate.argtypes = (
          c_char_p, c_wchar_p, c_int, POINTER(c_char), c_int)
 
+liblouis.lou_checkTable.argtypes = (c_char_p,)
+
 liblouis.lou_compileString.argtypes = (c_char_p, c_char_p)
 
+liblouis.lou_getTypeformForEmphClass.argtypes = (c_char_p, c_char_p)
+
 def version():
     """Obtain version information for liblouis.
     @return: The version of liblouis, plus other information, such as
@@ -99,7 +111,7 @@ def version():
     """
     return liblouis.lou_version().decode("ASCII")
 
-def translate(tableList, inbuf, typeform=None,cursorPos=0, mode=0):
+def translate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
     """Translate a string of characters, providing position information.
     @param tableList: A list of translation tables.
     @type tableList: list of str
@@ -127,19 +139,19 @@ def translate(tableList, inbuf, typeform=None,cursorPos=0, mode=0):
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if typeform:
-        typeformbuf = create_string_buffer(struct.pack('B'*len(typeform),*typeform), size=outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value, typeform)
     inPos = (c_int*outlen.value)()
     outPos = (c_int*inlen.value)()
     cursorPos = c_int(cursorPos)
     if not liblouis.lou_translate(tablesString, inbuf, byref(inlen), 
-                                  outbuf, byref(outlen),  typeformbuf, 
+                                  outbuf, byref(outlen), typeformbuf, 
                                   None, outPos, inPos, byref(cursorPos), mode):
-        raise RuntimeError("can't translate: tables %s, inbuf %s, typeform %s, cursorPos %s, mode %s"%(tableList, inbuf, typeform, cursorPos, mode))
+        raise RuntimeError("Can't translate: tables %s, inbuf %s, typeform %s, cursorPos %s, mode %s"%(tableList, inbuf, typeform, cursorPos, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value
+        typeform[:] = list(typeformbuf)
     return outbuf.value, inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
 
-def translateString(tableList, inbuf, typeform = None, mode = 0):
+def translateString(tableList, inbuf, typeform=None, mode=0):
     """Translate a string of characters.
     @param tableList: A list of translation tables.
     @type tableList: list of str
@@ -162,13 +174,13 @@ def translateString(tableList, inbuf, typeform = None, mode = 0):
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if typeform:
-        typeformbuf = create_string_buffer(struct.pack('B'*len(typeform),*typeform), size=outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value, typeform)
     if not liblouis.lou_translateString(tablesString, inbuf, byref(inlen), 
-                                        outbuf, byref(outlen),  typeformbuf, 
+                                        outbuf, byref(outlen), typeformbuf, 
                                         None, mode):
-        raise RuntimeError("can't translate: tables %s, inbuf %s, typeform %s, mode %s"%(tableList, inbuf, typeform, mode))
+        raise RuntimeError("Can't translate: tables %s, inbuf %s, typeform %s, mode %s"%(tableList, inbuf, typeform, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value
+        typeform[:] = list(typeformbuf)
     return outbuf.value
 
 def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
@@ -188,26 +200,26 @@ def backTranslate(tableList, inbuf, typeform=None, cursorPos=0, mode=0):
         a list of the output positions for each position in the input and
         the position of the cursor in the output.
     @rtype: (str, list of int, list of int, int)
-    @raises RuntimeError: If back translation could not be completed.
+    @raise RuntimeError: If a complete back translation could not be done.
     @see: lou_backTranslate in the liblouis documentation.
     """
-    tablestring = _createTablesString(tableList)
+    tablesString = _createTablesString(tableList)
     inbuf = createStr(inbuf)
     inlen = c_int(len(inbuf))
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if isinstance(typeform, list):
-        typeformbuf = create_string_buffer(outlen.value)
+        typeformbuf = _createTypeformbuf(outlen.value)
     inPos = (c_int*outlen.value)()
     outPos = (c_int*inlen.value)()
     cursorPos = c_int(cursorPos)
-    if not liblouis.lou_backTranslate(tablestring, inbuf, byref(inlen),
+    if not liblouis.lou_backTranslate(tablesString, inbuf, byref(inlen),
                     outbuf, byref(outlen), typeformbuf, None,
                     outPos, inPos, byref(cursorPos), mode):
-        raise RuntimeError("Can't back translate tableList %s, inbuf %s, typeform %s, cursorPos %d, mode %d" % (tableList, inbuf, typeform, cursorPos, mode))
+        raise RuntimeError("Can't back translate: tables %s, inbuf %s, typeform %s, cursorPos %d, mode %d"%(tableList, inbuf, typeform, cursorPos, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value
+        typeform[:] = list(typeformbuf)
     return outbuf.value, inPos[:outlen.value], outPos[:inlen.value], cursorPos.value
 
 def backTranslateString(tableList, inbuf, typeform=None, mode=0):
@@ -223,22 +235,22 @@ def backTranslateString(tableList, inbuf, typeform=None, mode=0):
     @type mode: int
     @return: The back translation of inbuf.
     @rtype: str
-    @raises RuntimeError: If a full back translation could not be done.
+    @raise RuntimeError: If a complete back translation could not be done.
     @see: lou_backTranslateString in the liblouis documentation.
     """
-    tablestring = _createTablesString(tableList)
+    tablesString = _createTablesString(tableList)
     inbuf = createStr(inbuf)
     inlen = c_int(len(inbuf))
     outlen = c_int(inlen.value * outlenMultiplier)
     outbuf = create_unicode_buffer(outlen.value)
     typeformbuf = None
     if isinstance(typeform, list):
-        typeformbuf = create_string_buffer(outlen.value)
-    if not liblouis.lou_backTranslateString(tablestring, inbuf, byref(inlen), outbuf,
+        typeformbuf = _createTypeformbuf(outlen.value)
+    if not liblouis.lou_backTranslateString(tablesString, inbuf, byref(inlen), outbuf,
                     byref(outlen), typeformbuf, None, mode):
-        raise RuntimeError("Can't back translate tables %s, inbuf %s, mode %d" %(tablestring, inbuf, mode))
+        raise RuntimeError("Can't back translate: tables %s, inbuf %s, mode %d"%(tableList, inbuf, mode))
     if isinstance(typeform, list):
-        typeform[:] = typeformbuf.value[:outlen.value]
+        typeform[:] = list(typeformbuf)
     return outbuf.value
 
 def hyphenate(tableList, inbuf, mode=0):
@@ -256,7 +268,7 @@ def hyphenate(tableList, inbuf, mode=0):
     @return: A string with '1' at the beginning of every syllable
         and '0' elsewhere.
     @rtype: str
-    @raises RuntimeError: If hyphenation data could not be produced.
+    @raise RuntimeError: If hyphenation data could not be produced.
     @see: lou_hyphenate in the liblouis documentation.
     """
     tablesString = _createTablesString(tableList)
@@ -264,9 +276,23 @@ def hyphenate(tableList, inbuf, mode=0):
     inlen = c_int(len(inbuf))
     hyphen_string = create_string_buffer(inlen.value + 1) 
     if not liblouis.lou_hyphenate(tablesString, inbuf, inlen, hyphen_string, mode):
-        raise RuntimeError("Can't hyphenate tables %s, inbuf %s, mode %d" %(tablesString, inbuf, mode))
+        raise RuntimeError("Can't hyphenate: tables %s, inbuf %s, mode %d"%(tableList, inbuf, mode))
     return hyphen_string.value.decode("ASCII")
 
+def checkTable(tableList):
+    """Check if the specified tables can be found and compiled.
+        This can be used to check if a list of tables contains errors
+        before sending it to other liblouis functions
+        that accept a list of tables.
+    @param tableList: A list of translation tables.
+    @type tableList: list of str
+    @raise RuntimeError: If compilation failed.
+    @see: lou_checkTable in the liblouis documentation
+    """
+    tablesString = _createTablesString(tableList)
+    if not liblouis.lou_checkTable(tablesString):
+        raise RuntimeError("Can't compile: tables %s"%tableList)
+
 def compileString(tableList, inString):
     """Compile a table entry on the fly at run-time.
     @param tableList: A list of translation tables.
@@ -279,14 +305,35 @@ def compileString(tableList, inString):
     tablesString = _createTablesString(tableList)
     inBytes = inString.encode("ASCII") if isinstance(inString, str) else bytes(inString)
     if not liblouis.lou_compileString(tablesString, inString):
-        raise RuntimeError("Can't compile entry: tables %s, inString %s" % (tableList, inString))
+        raise RuntimeError("Can't compile entry: tables %s, inString %s"%(tableList, inString))
+
+def getTypeformForEmphClass(tableList, emphClass):
+    """Get the typeform bit for the named emphasis class.
+    @param tableList: A list of translation tables.
+    @type tableList: list of str
+    @param emphClass: An emphasis class name.
+    @type emphClass: str
+    @see: lou_getTypeformForEmphClass in the liblouis documentation
+    """
+    tablesString = _createTablesString(tableList)
+    return liblouis.lou_getTypeformForEmphClass(tablesString, emphClass)
 
 #{ Typeforms
-plain_text = 0
-italic = 1
-underline = 2
-bold = 4
-computer_braille = 8
+plain_text = 0x0000
+italic = 0x0001       # emph_1
+underline = 0x0002    # emph_2
+bold = 0x0004         # emph_3
+emph_4 = 0x0008
+emph_5 = 0x0010
+emph_6 = 0x0020
+emph_7 = 0x0040
+emph_8 = 0x0080
+emph_9 = 0x0100
+emph_10 = 0x0200
+computer_braille = 0x0400
+no_translate = 0x0800
+no_contract = 0x1000
+#}
 
 #{ Translation modes
 noContractions = 1
@@ -303,5 +350,3 @@ if __name__ == '__main__':
     # Just some common tests.
     print(version())
     print(translate([b'../tables/en-us-g2.ctb'], 'Hello world!', cursorPos=5))
-
-
